/*
 * This is a basic stand-alone program to do image processing
 * with DIPlib and dipIO.
 *
 * The error handling is implemented through macros. standalone2.c
 * is the same program as this, but without the error handling
 * macros. This makes the whole thing somewhat more complex, but
 * illustrates the error handling.
 *
 * Cris Luengo, June 2000, March 2002, September 2006
 */

/*
gcc -o standalone standalone.c         \
    -I/home/diplib/dip/sunOS/include   \
    -L/home/diplib/dip/sunOS/lib       \
    -ldipio -ldip -lm
*/

#include <stdio.h> /* for stderr */
#include <string.h> /* for strcpy */
#include "diplib.h" /* Include for basic DIPlib functions */
#include "dipio.h" /* Include to use dipIO */

#include "dip_linear.h" /* For dip_Gauss */
#include "dipio_tiff.h" /* For dipio_WriteTIFFID */

int main (void) {

  DIP_FN_DECLARE("main");
  dip_Resources rg = 0;
  dip_Image im;
  dip_String filename;
  dip_FloatArray sigmas;
  dipio_Compression comp;

  /* Initialize both DIPlib and dipIO */
  DIPXJ( dip_Initialise () );
  DIPXJ( dipio_Initialise () );

  /* Allocate Resources structure to keep track of allocated memory */
  DIPXJ( dip_ResourcesNew (&rg, 0) );

  /* This is the image we're going to use */
  DIPXJ( dip_ImageNew (&im, rg) );

  /* Allocate string with 1024 characters and fill with filename */
  DIPXJ( dip_StringNew (&filename, 1024, "orka.tif", rg) );

  /* Create 2D array with values [5,5] for the filter parameter */
  DIPXJ( dip_FloatArrayNew (&sigmas, 2, 5., rg) );

  /* Get the image from the file */
  DIPXJ( dipio_ImageRead (im, filename, 0, DIP_FALSE, 0) );

  /* Display some data on the image read */
  {
    dip_IntegerArray dimensions;
    dip_ImageType type;
    dip_DataType dataType;
    dip_int ii;

    DIPXJ( dip_ImageGetDimensions (im, &dimensions, rg) );
    DIPXJ( dip_ImageGetType (im, &type) );
    DIPXJ( dip_ImageGetDataType (im, &dataType) );

    printf ("\nImage read:\nNumber of dimensions: %d\n",
      dimensions->size);
    if (dimensions->size > 0) {
      printf ("Dimensions: [%d", dimensions->array[0]);
      for (ii = 1; ii < dimensions->size; ii++)
        printf (", %d", dimensions->array[ii]);
      printf ("]\n");
    }
    printf ("Image type: %s\nData type: %s\n\n",
      type == DIP_IMTP_SCALAR ? "scalar" : "unknown",
      dataType == DIP_DT_UINT8 ? "uint8" : "not uint8");
  }

  /* This is the image processing */
  DIPXJ( dip_Gauss (im, im, 0, 0, sigmas, 0, -1) );

  /* dip_Gauss always returns floating point types. We want uint8
     because most programs don't recognize floating-point TIFFs. */
  DIPXJ( dip_ConvertDataType (im, im, DIP_DT_UINT8) );

  /* The allocated space in 'filename' is large enough for this */
  strcpy (filename->string, "orka_smooth.tif");

  /* Choose a compression method and level */
  comp.method = DIPIO_CMP_DEFLATE;
  comp.level = 6;
  
  /* Write the image to another file */
  DIPXJ( dipio_ImageWrite (im, filename, 0, dipio_WriteTIFFID(), comp) );

dip_error:

  /* Return all allocated memory to the system */
  DIPXC( dip_ResourcesFree (&rg) );

  /* Let libraries clean up after themselves */
  DIPXC( dipio_Exit () );
  DIPXC( dip_Exit () );

  /* Report errors */
  return dip_ErrorWrite( error, 0, 0, stderr);
}
